@@ -8,6 +8,7 @@ from typing import Optional, Union, Mapping  # Special
 from typing import Sequence, Iterable  # ABCs
 from typing import Tuple  # Classes
 
+import numbers
 import numpy as np
 import pandas as pd
 from anndata import AnnData
@@ -20,7 +21,7 @@ import networkx as nx
 
 from ..classes import Spatial
 from ..utils import _AxesSubplot, Axes, _read_graph
-from .utils import centroidpython, get_cluster, get_node, check_sublist
+from .utils import centroidpython, get_cluster, get_node, check_sublist, get_cmap
 
 ################################################################
 #                                                              #
@@ -39,11 +40,13 @@ class SpatialBasePlot(Spatial):
         cmap: Optional[str] = "Spectral_r",
         use_label: Optional[str] = None,
         list_clusters: Optional[list] = None,
-        ax: Optional[_AxesSubplot] = None,
+        ax: Optional[matplotlib.axes._subplots.Axes] = None,
+        fig: Optional[matplotlib.figure.Figure] = None,
         show_plot: Optional[bool] = True,
         show_axis: Optional[bool] = False,
         show_image: Optional[bool] = True,
         show_color_bar: Optional[bool] = True,
+        color_bar_label: Optional[str] = "",
         crop: Optional[bool] = True,
         margin: Optional[bool] = 100,
         size: Optional[float] = 7,
@@ -52,7 +55,7 @@ class SpatialBasePlot(Spatial):
         use_raw: Optional[bool] = False,
         fname: Optional[str] = None,
         dpi: Optional[int] = 120,
-        **kwds
+        **kwds,
     ):
         super().__init__(
             adata,
@@ -89,7 +92,13 @@ class SpatialBasePlot(Spatial):
                 if type(self.list_clusters) != list:
                     self.list_clusters = [self.list_clusters]
 
-                self.list_clusters = np.array(self.list_clusters).astype(str)
+                clusters_indexes = [
+                    np.where(adata.obs[use_label].cat.categories == i)[0][0]
+                    for i in self.list_clusters
+                ]
+                self.list_clusters = np.array(self.list_clusters)[
+                    np.argsort(clusters_indexes)
+                ]
 
             self.query_indexes = self._get_query_clusters_index()
 
@@ -99,12 +108,20 @@ class SpatialBasePlot(Spatial):
         scanpy_cmap = ["vega_10_scanpy", "vega_20_scanpy", "default_102", "default_28"]
         stlearn_cmap = ["jana_40", "default"]
         cmap_available = plt.colormaps() + scanpy_cmap + stlearn_cmap
-        assert cmap in cmap_available, "cmap should be one of these colors: " + str(
-            cmap_available
+        error_msg = (
+            "cmap must be a matplotlib.colors.LinearSegmentedColormap OR"
+            "one of these: " + str(cmap_available)
         )
+        if type(cmap) == str:
+            assert cmap in cmap_available, error_msg
+        elif type(cmap) != matplotlib.colors.LinearSegmentedColormap:
+            raise Exception(error_msg)
         self.cmap = cmap
 
-        self.fig, self.ax = self._generate_frame()
+        if type(fig) == type(None) and type(ax) == type(None):
+            self.fig, self.ax = self._generate_frame()
+        else:
+            self.fig, self.ax = fig, ax
 
         if show_axis == False:
             self._remove_axis(self.ax)
@@ -112,8 +129,10 @@ class SpatialBasePlot(Spatial):
         if show_image:
             self._add_image(self.ax)
 
-        if show_plot == False:
-            plt.close(self.fig)
+        ## Don't want to do this, since then can't build on figure when parsing
+        ## fig, ax as input
+        # if show_plot == False:
+        #     plt.close(self.fig)
 
     def _select_clusters(self):
         def create_query(list_cl, use_label):
@@ -123,11 +142,17 @@ class SpatialBasePlot(Spatial):
             return ini[:-2]
 
         if self.list_clusters is not None:
-            self.query_adata = self.query_adata[
-                self.query_adata.obs.query(
-                    create_query(self.list_clusters, self.use_label)
-                ).index
-            ].copy()
+            # IF not all clusters specified, subset, otherwise just copy.
+            if len(self.list_clusters) != len(
+                self.adata[0].obs[self.use_label].cat.categories
+            ):
+                self.query_adata = self.query_adata[
+                    self.query_adata.obs.query(
+                        create_query(self.list_clusters, self.use_label)
+                    ).index
+                ].copy()
+            else:
+                self.query_adata = self.query_adata.copy()
         else:
             self.query_adata = self.query_adata.copy()
 
@@ -145,9 +170,11 @@ class SpatialBasePlot(Spatial):
             zorder=-1,
         )
 
-    def _plot_colorbar(self, plot_ax: Axes):
+    def _plot_colorbar(self, plot_ax: Axes, color_bar_label: str = ""):
 
-        cb = plt.colorbar(plot_ax, aspect=10, shrink=0.5, cmap=self.cmap)
+        cb = plt.colorbar(
+            plot_ax, aspect=10, shrink=0.5, cmap=self.cmap, label=color_bar_label
+        )
         cb.outline.set_visible(False)
 
     def _remove_axis(self, main_ax: Axes):
@@ -161,8 +188,10 @@ class SpatialBasePlot(Spatial):
 
         main_ax.set_ylim(main_ax.get_ylim()[::-1])
 
-    def _add_color_bar(self, plot):
-        cb = plt.colorbar(plot, aspect=10, shrink=0.5, cmap=self.cmap)
+    def _add_color_bar(self, plot, color_bar_label: str = ""):
+        cb = plt.colorbar(
+            plot, aspect=10, shrink=0.5, cmap=self.cmap, label=color_bar_label
+        )
         cb.outline.set_visible(False)
 
     def _add_title(self):
@@ -203,11 +232,13 @@ class GenePlot(SpatialBasePlot):
         cmap: Optional[str] = "Spectral_r",
         use_label: Optional[str] = None,
         list_clusters: Optional[list] = None,
-        ax: Optional[_AxesSubplot] = None,
+        ax: Optional[matplotlib.axes._subplots.Axes] = None,
+        fig: Optional[matplotlib.figure.Figure] = None,
         show_plot: Optional[bool] = True,
         show_axis: Optional[bool] = False,
         show_image: Optional[bool] = True,
         show_color_bar: Optional[bool] = True,
+        color_bar_label: Optional[str] = "",
         crop: Optional[bool] = True,
         margin: Optional[bool] = 100,
         size: Optional[float] = 7,
@@ -222,7 +253,9 @@ class GenePlot(SpatialBasePlot):
         method: str = "CumSum",
         contour: bool = False,
         step_size: Optional[int] = None,
-        **kwargs
+        vmin: float = None,
+        vmax: float = None,
+        **kwargs,
     ):
         super().__init__(
             adata=adata,
@@ -232,6 +265,7 @@ class GenePlot(SpatialBasePlot):
             use_label=use_label,
             list_clusters=list_clusters,
             ax=ax,
+            fig=fig,
             show_plot=show_plot,
             show_axis=show_axis,
             show_image=show_image,
@@ -270,13 +304,15 @@ class GenePlot(SpatialBasePlot):
 
         self.available_ids = self._add_threshold(gene_values, threshold)
 
+        self.vmin, self.vmax = vmin, vmax
+
         if contour:
             plot = self._plot_contour(gene_values[self.available_ids])
         else:
             plot = self._plot_genes(gene_values[self.available_ids])
 
         if show_color_bar:
-            self._add_color_bar(plot)
+            self._add_color_bar(plot, color_bar_label=color_bar_label)
 
         if crop:
             self._crop_image(self.ax, margin)
@@ -333,8 +369,12 @@ class GenePlot(SpatialBasePlot):
 
     def _plot_genes(self, gene_values: pd.Series):
 
-        vmin = min(gene_values)
-        vmax = max(gene_values)
+        if type(self.vmin) == type(None) and type(self.vmax) == type(None):
+            vmin = min(gene_values)
+            vmax = max(gene_values)
+        else:
+            vmin, vmax = self.vmin, self.vmax
+
         # Plot scatter plot based on pixel of spots
         imgcol_new = self.query_adata.obsm["spatial"][:, 0] * self.scale_factor
         imgrow_new = self.query_adata.obsm["spatial"][:, 1] * self.scale_factor
@@ -347,7 +387,7 @@ class GenePlot(SpatialBasePlot):
             marker="o",
             vmin=vmin,
             vmax=vmax,
-            cmap=plt.get_cmap(self.cmap),
+            cmap=plt.get_cmap(self.cmap) if type(self.cmap) == str else self.cmap,
             c=gene_values,
         )
         return plot
@@ -370,14 +410,14 @@ class GenePlot(SpatialBasePlot):
             self.step_size = int(np.max(z) / 50)
             if self.step_size < 1:
                 self.step_size = 1
-        # Creating contour plot with a step size of 10
+        # Creating contour plot with a step size of 1
 
         cs = plt.contourf(
             xi,
             yi,
             zi,
             range(0, int(np.nanmax(zi)) + self.step_size, self.step_size),
-            cmap=plt.get_cmap(self.cmap),
+            cmap=plt.get_cmap(self.cmap) if type(self.cmap) == str else self.cmap,
             alpha=self.cell_alpha,
         )
         return cs
@@ -389,6 +429,177 @@ class GenePlot(SpatialBasePlot):
             return gene_values > threshold
 
 
+################################################################
+#                                                              #
+#                      Feature plot class                      #
+#                                                              #
+################################################################
+
+
+class FeaturePlot(SpatialBasePlot):
+    def __init__(
+        self,
+        adata: AnnData,
+        # plotting param
+        title: Optional["str"] = None,
+        figsize: Optional[Tuple[float, float]] = None,
+        cmap: Optional[str] = "Spectral_r",
+        use_label: Optional[str] = None,
+        list_clusters: Optional[list] = None,
+        ax: Optional[matplotlib.axes._subplots.Axes] = None,
+        fig: Optional[matplotlib.figure.Figure] = None,
+        show_plot: Optional[bool] = True,
+        show_axis: Optional[bool] = False,
+        show_image: Optional[bool] = True,
+        show_color_bar: Optional[bool] = True,
+        color_bar_label: Optional[str] = "",
+        crop: Optional[bool] = True,
+        margin: Optional[bool] = 100,
+        size: Optional[float] = 7,
+        image_alpha: Optional[float] = 1.0,
+        cell_alpha: Optional[float] = 1.0,
+        use_raw: Optional[bool] = False,
+        fname: Optional[str] = None,
+        dpi: Optional[int] = 120,
+        # gene plot param
+        feature: str = None,
+        threshold: Optional[float] = None,
+        contour: bool = False,
+        step_size: Optional[int] = None,
+        vmin: float = None,
+        vmax: float = None,
+        **kwargs,
+    ):
+        super().__init__(
+            adata=adata,
+            title=title,
+            figsize=figsize,
+            cmap=cmap,
+            use_label=use_label,
+            list_clusters=list_clusters,
+            ax=ax,
+            fig=fig,
+            show_plot=show_plot,
+            show_axis=show_axis,
+            show_image=show_image,
+            show_color_bar=show_color_bar,
+            crop=crop,
+            margin=margin,
+            size=size,
+            image_alpha=image_alpha,
+            cell_alpha=cell_alpha,
+            use_raw=use_raw,
+            fname=fname,
+            dpi=dpi,
+        )
+
+        self.step_size = step_size
+
+        self.title = feature
+        self._add_title()
+
+        self.feature = feature
+
+        feature_values = self._get_feature_values()
+
+        self.available_ids = self._add_threshold(feature_values, threshold)
+
+        self.vmin, self.vmax = vmin, vmax
+
+        if contour:
+            plot = self._plot_contour(feature_values[self.available_ids])
+        else:
+            plot = self._plot_feature(feature_values[self.available_ids])
+
+        if show_color_bar:
+            self._add_color_bar(plot, color_bar_label=color_bar_label)
+
+        if crop:
+            self._crop_image(self.ax, margin)
+
+        if fname != None:
+            self._save_output()
+
+    def _get_feature_values(self):
+
+        if self.feature not in self.query_adata.obs:
+            raise ValueError(
+                self.feature + " is not in data.obs, please try another feature"
+            )
+        elif not isinstance(
+            self.query_adata.obs[self.feature].values[0], numbers.Number
+        ):
+            raise ValueError(
+                self.feature
+                + " in data.obs is not continuous, please try another feature"
+            )
+
+        colors = self.query_adata.obs[self.feature]
+
+        return colors
+
+    def _plot_feature(self, feature_values: pd.Series):
+
+        if type(self.vmin) == type(None) and type(self.vmax) == type(None):
+            vmin = min(feature_values)
+            vmax = max(feature_values)
+        else:
+            vmin, vmax = self.vmin, self.vmax
+
+        # Plot scatter plot based on pixel of spots
+        imgcol_new = self.query_adata.obsm["spatial"][:, 0] * self.scale_factor
+        imgrow_new = self.query_adata.obsm["spatial"][:, 1] * self.scale_factor
+        plot = self.ax.scatter(
+            imgcol_new,
+            imgrow_new,
+            edgecolor="none",
+            alpha=self.cell_alpha,
+            s=self.size,
+            marker="o",
+            vmin=vmin,
+            vmax=vmax,
+            cmap=plt.get_cmap(self.cmap) if type(self.cmap) == str else self.cmap,
+            c=feature_values,
+        )
+        return plot
+
+    def _plot_contour(self, feature_values: pd.Series):
+
+        imgcol_new = self.query_adata.obsm["spatial"][:, 0] * self.scale_factor
+        imgrow_new = self.query_adata.obsm["spatial"][:, 1] * self.scale_factor
+        # Extracting x,y and values (z)
+        z = feature_values
+        y = imgrow_new
+        x = imgcol_new
+
+        # Interpolating values to get better coverage
+        xi = np.linspace(x.min(), x.max(), 100)
+        yi = np.linspace(y.min(), y.max(), 100)
+        zi = griddata((x, y), z, (xi[None, :], yi[:, None]), method="linear")
+
+        if self.step_size == None:
+            self.step_size = int(np.max(z) / 50)
+            if self.step_size < 1:
+                self.step_size = 1
+        # Creating contour plot with a step size of 1
+
+        cs = plt.contourf(
+            xi,
+            yi,
+            zi,
+            range(0, int(np.nanmax(zi)) + self.step_size, self.step_size),
+            cmap=plt.get_cmap(self.cmap) if type(self.cmap) == str else self.cmap,
+            alpha=self.cell_alpha,
+        )
+        return cs
+
+    def _add_threshold(self, feature_values, threshold):
+        if threshold == None:
+            return np.repeat(True, len(feature_values))
+        else:
+            return feature_values > threshold
+
+
 ################################################################
 #                                                              #
 #                      Cluster plot class                      #
@@ -406,7 +617,8 @@ class ClusterPlot(SpatialBasePlot):
         cmap: Optional[str] = "default",
         use_label: Optional[str] = None,
         list_clusters: Optional[list] = None,
-        ax: Optional[_AxesSubplot] = None,
+        ax: Optional[matplotlib.axes._subplots.Axes] = None,
+        fig: Optional[matplotlib.figure.Figure] = None,
         show_plot: Optional[bool] = True,
         show_axis: Optional[bool] = False,
         show_image: Optional[bool] = True,
@@ -437,6 +649,7 @@ class ClusterPlot(SpatialBasePlot):
             use_label=use_label,
             list_clusters=list_clusters,
             ax=ax,
+            fig=fig,
             show_plot=show_plot,
             show_axis=show_axis,
             show_image=show_image,
@@ -479,28 +692,43 @@ class ClusterPlot(SpatialBasePlot):
             self._save_output()
 
     def _add_cluster_colors(self):
-        self.adata[0].uns[self.use_label + "_colors"] = []
+        if self.use_label + "_colors" not in self.adata[0].uns:
+            # self.adata[0].uns[self.use_label + "_set"] = []
+            self.adata[0].uns[self.use_label + "_colors"] = []
 
-        for i, cluster in enumerate(self.adata[0].obs.groupby(self.use_label)):
-            self.adata[0].uns[self.use_label + "_colors"].append(
-                matplotlib.colors.to_hex(self.cmap_(i / (self.cmap_n - 1)))
-            )
+            for i, cluster in enumerate(self.adata[0].obs.groupby(self.use_label)):
+                self.adata[0].uns[self.use_label + "_colors"].append(
+                    matplotlib.colors.to_hex(self.cmap_(i / (self.cmap_n - 1)))
+                )
+                # self.adata[0].uns[self.use_label + "_set"].append( cluster[0] )
 
     def _plot_clusters(self):
         # Plot scatter plot based on pixel of spots
 
+        # for i, cluster in enumerate(self.query_adata.obs[self.use_label].cat.categories):
         for i, cluster in enumerate(self.query_adata.obs.groupby(self.use_label)):
+
             # Plot scatter plot based on pixel of spots
             subset_spatial = self.query_adata.obsm["spatial"][
                 check_sublist(list(self.query_adata.obs.index), list(cluster[1].index))
             ]
 
+            if self.use_label + "_colors" in self.adata[0].uns:
+                # label_set = self.adata[0].uns[self.use_label+'_set']
+                label_set = (
+                    self.adata[0].obs[self.use_label].cat.categories.values.astype(str)
+                )
+                col_index = np.where(label_set == cluster[0])[0][0]
+                color = self.adata[0].uns[self.use_label + "_colors"][col_index]
+            else:
+                color = self.cmap_(self.query_indexes[i] / (self.cmap_n - 1))
+
             imgcol_new = subset_spatial[:, 0] * self.scale_factor
             imgrow_new = subset_spatial[:, 1] * self.scale_factor
             _ = self.ax.scatter(
                 imgcol_new,
                 imgrow_new,
-                c=[self.cmap_(self.query_indexes[i] / (self.cmap_n - 1))],
+                c=[color],
                 label=cluster[0],
                 edgecolor="none",
                 alpha=self.cell_alpha,
@@ -509,29 +737,8 @@ class ClusterPlot(SpatialBasePlot):
             )
 
     def _get_cmap(self, cmap):
-        from scanpy.plotting import palettes
-        from stlearn.plotting import palettes_st
-
-        if cmap == "vega_10_scanpy":
-            cmap = palettes.vega_10_scanpy
-        elif cmap == "vega_20_scanpy":
-            cmap = palettes.vega_20_scanpy
-        elif cmap == "default_102":
-            cmap = palettes.default_102
-        elif cmap == "default_28":
-            cmap = palettes.default_28
-        elif cmap == "jana_40":
-            cmap = palettes_st.jana_40
-        elif cmap == "default":
-            cmap = palettes_st.default
-        else:
-            self.cmap_n = plt.get_cmap(cmap).N
-            return plt.get_cmap(cmap)
-
-        self.cmap_n = len(cmap)
-        cmaps = matplotlib.colors.LinearSegmentedColormap.from_list("", cmap)
-
-        cmap_ = plt.cm.get_cmap(cmaps)
+        cmap_, cmap_n = get_cmap(cmap)
+        self.cmap_n = cmap_n
         return cmap_
 
     def _add_cluster_bar(self, bbox_to_anchor):
@@ -768,7 +975,8 @@ class SubClusterPlot(SpatialBasePlot):
         cmap: Optional[str] = "jet",
         use_label: Optional[str] = None,
         list_clusters: Optional[list] = None,
-        ax: Optional[_AxesSubplot] = None,
+        ax: Optional[matplotlib.axes._subplots.Axes] = None,
+        fig: Optional[matplotlib.figure.Figure] = None,
         show_plot: Optional[bool] = True,
         show_axis: Optional[bool] = False,
         show_image: Optional[bool] = True,
@@ -784,7 +992,7 @@ class SubClusterPlot(SpatialBasePlot):
         cluster: Optional[int] = 0,
         text_box_size: Optional[float] = 5,
         bbox_to_anchor: Optional[Tuple[float, float]] = (1, 1),
-        **kwargs
+        **kwargs,
     ):
         super().__init__(
             adata=adata,
@@ -794,6 +1002,7 @@ class SubClusterPlot(SpatialBasePlot):
             use_label=use_label,
             list_clusters=list_clusters,
             ax=ax,
+            fig=fig,
             show_plot=show_plot,
             show_axis=show_axis,
             show_image=show_image,
@@ -841,7 +1050,7 @@ class SubClusterPlot(SpatialBasePlot):
             edgecolor="none",
             s=self.size,
             marker="o",
-            cmap=plt.get_cmap(self.cmap),
+            cmap=plt.get_cmap(self.cmap) if type(self.cmap) == str else self.cmap,
             c=colors,
             alpha=self.cell_alpha,
         )
@@ -932,7 +1141,8 @@ class CciPlot(GenePlot):
         cmap: Optional[str] = "Spectral_r",
         use_label: Optional[str] = None,
         list_clusters: Optional[list] = None,
-        ax: Optional[_AxesSubplot] = None,
+        ax: Optional[matplotlib.axes._subplots.Axes] = None,
+        fig: Optional[matplotlib.figure.Figure] = None,
         show_plot: Optional[bool] = True,
         show_axis: Optional[bool] = False,
         show_image: Optional[bool] = True,
@@ -945,20 +1155,22 @@ class CciPlot(GenePlot):
         use_raw: Optional[bool] = False,
         fname: Optional[str] = None,
         dpi: Optional[int] = 120,
-        # cci param
+        # cci_rank param
         use_het: Optional[str] = "het",
         contour: bool = False,
         step_size: Optional[int] = None,
-        **kwargs
+        vmin: float = None,
+        vmax: float = None,
+        **kwargs,
     ):
         super().__init__(
             adata=adata,
-            title=title,
             figsize=figsize,
             cmap=cmap,
             use_label=use_label,
             list_clusters=list_clusters,
             ax=ax,
+            fig=fig,
             show_plot=show_plot,
             show_axis=show_axis,
             show_image=show_image,
@@ -974,7 +1186,103 @@ class CciPlot(GenePlot):
             gene_symbols=use_het,
             contour=contour,
             step_size=step_size,
+            vmin=vmin,
+            vmax=vmax,
         )
 
+        self.title = title
+
+        self._add_title()
+
     def _get_gene_expression(self):
         return self.query_adata.obsm[self.gene_symbols[0]]
+
+
+class LrResultPlot(GenePlot):
+    def __init__(
+        self,
+        adata: AnnData,
+        use_lr: Optional["str"] = None,
+        use_result: Optional["str"] = "lr_sig_scores",
+        # plotting param
+        title: Optional["str"] = None,
+        figsize: Optional[Tuple[float, float]] = None,
+        cmap: Optional[str] = "Spectral_r",
+        list_clusters: Optional[list] = None,
+        ax: Optional[matplotlib.axes._subplots.Axes] = None,
+        fig: Optional[matplotlib.figure.Figure] = None,
+        show_plot: Optional[bool] = True,
+        show_axis: Optional[bool] = False,
+        show_image: Optional[bool] = True,
+        show_color_bar: Optional[bool] = True,
+        crop: Optional[bool] = True,
+        margin: Optional[bool] = 100,
+        size: Optional[float] = 7,
+        image_alpha: Optional[float] = 1.0,
+        cell_alpha: Optional[float] = 1.0,
+        use_raw: Optional[bool] = False,
+        fname: Optional[str] = None,
+        dpi: Optional[int] = 120,
+        # cci_rank param
+        contour: bool = False,
+        step_size: Optional[int] = None,
+        vmin: float = None,
+        vmax: float = None,
+        **kwargs,
+    ):
+        # Making sure cci_rank has been run first #
+        if "lr_summary" not in adata.uns:
+            raise Exception(
+                f"To visualise LR interaction results, must run" f"st.pl.cci.run first."
+            )
+
+        # By default, using the LR with most significant spots #
+        if type(use_lr) == type(None):
+            use_lr = adata.uns["lr_summary"].index.values[0]
+        elif use_lr not in adata.uns["lr_summary"].index:
+            raise Exception(
+                f"use_lr must be one of:\n" f'{adata.uns["lr_summary"].index}'
+            )
+        else:
+            use_lr = str(use_lr)
+
+        # Checking is a valid result #
+        res_info = ["lr_scores", "p_vals", "p_adjs", "-log10(p_adjs)", "lr_sig_scores"]
+        if use_result not in res_info:
+            raise Exception(f"use_result must be one of:\n{res_info}")
+        else:
+            self.use_result = use_result
+
+        super().__init__(
+            adata=adata,
+            title=title,
+            figsize=figsize,
+            cmap=cmap,
+            list_clusters=list_clusters,
+            ax=ax,
+            fig=fig,
+            show_plot=show_plot,
+            show_axis=show_axis,
+            show_image=show_image,
+            show_color_bar=show_color_bar,
+            crop=crop,
+            margin=margin,
+            size=size,
+            image_alpha=image_alpha,
+            cell_alpha=cell_alpha,
+            use_raw=use_raw,
+            fname=fname,
+            dpi=dpi,
+            gene_symbols=use_lr,
+            contour=contour,
+            step_size=step_size,
+            vmin=vmin,
+            vmax=vmax,
+        )
+
+    def _get_gene_expression(self):
+        use_lr = self.gene_symbols[0]
+        index = np.where(self.query_adata.uns["lr_summary"].index.values == use_lr)[0][
+            0
+        ]
+        return self.query_adata.obsm[self.use_result][:, index]
